/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.desktop.about;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;

import nu.zoom.gal.error.ErrorReporter;
import nu.zoom.swing.desktop.PlugIn;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchFrame;
import nu.zoom.swing.desktop.WorkbenchListener;
import nu.zoom.swing.desktop.common.action.ToggleFrameAction;
import nu.zoom.swing.desktop.common.action.WorkbenchAction;
import nu.zoom.ui.Resources;
import nu.zoom.ui.Resources.ResourceNotFoundException;

/**
 * zoom.nu desktop plug-in for displaying the Svansprogram about dialog.
 * 
 * @author "Johan Maasing" &lt;johan@zoom.nu&gt;
 */
public class AboutPlugIn implements PlugIn, WorkbenchListener {

	private final Workbench workbench;
	private final Resources messages;
	private final ErrorReporter errorReporter;
	private String version = "?";
	private AboutComponent aboutComponent;

	public AboutPlugIn(final Workbench workbench, final Resources messages,
			final ErrorReporter errorReporter) {
		super();
		this.workbench = workbench;
		this.messages = messages;
		this.errorReporter = errorReporter;
	}

	@Override
	public void initialize() throws Exception {
		workbench.addWorkBenchListener(this);
	}

	@Override
	public void close() {
	}

	@Override
	public void start() {
		aboutComponent = new AboutComponent(errorReporter, messages,
				AboutPlugIn.this.version);
		aboutComponent.init();
		workbench.getMenuBar().addToHelpMenu(
				new AboutMenuItem(messages).getControl());
		workbench.getMenuBar().addToHelpMenu(
				new JMenuItem(new OnlineHelpItem()));
		try {
			URL resource = getClass()
					.getResource(
							"/META-INF/maven/nu.zoom.svansprogram/svansprogram-about/pom.properties");
			if (resource != null) {
				Properties p = new Properties();
				p.load(resource.openStream());
				String versionProperty = p.getProperty("version");
				if (versionProperty != null) {
					this.version = versionProperty;
				}
			}
		} catch (IOException e) {
		}
	}

	class OnlineHelpItem extends WorkbenchAction {
		private static final long serialVersionUID = 2861212265555466730L;

		public OnlineHelpItem() {
			super();
			try {
				setName(messages
						.getMessage("nu.zoom.catonine.about.onlinehelp"));
				setIcon(messages
						.getIcon("nu.zoom.catonine.about.onlinehelp.icon"));
			} catch (ResourceNotFoundException e) {
				errorReporter.reportError(e);
			}
		}

		@Override
		public void actionPerformed(final ActionEvent evnt) {
			try {
				final String urlString = messages
						.getMessage("nu.zoom.catonine.about.onlinehelp.url");
				aboutComponent.openBrowser(urlString);
			} catch (final ResourceNotFoundException e) {
				errorReporter.reportError(e);
			}
		}

	}

	class AboutMenuItem extends ToggleFrameAction<JCheckBoxMenuItem> {
		private static final String FRAME_PREFERENCE_KEY = "catonine.about";
		private static final long serialVersionUID = 1L;

		public AboutMenuItem(Resources messages) {
			super(new JCheckBoxMenuItem());
			try {
				setName(messages.getMessage("nu.zoom.catonine.about.menuitem"));
			} catch (ResourceNotFoundException ex) {
				errorReporter.reportError(ex);
			}
		}

		@Override
		protected WorkbenchFrame createFrame() {
			WorkbenchFrame frame = workbench.createWorkbenchFrame(
					FRAME_PREFERENCE_KEY, aboutComponent, null, true, true);
			try {
				frame.setTitle(messages
						.getMessage("nu.zoom.catonine.about.menuitem"));
			} catch (ResourceNotFoundException ex) {
				errorReporter.reportError(ex);
			}
			return frame;
		}

		@Override
		public void frameDetached(WorkbenchFrame frame) {
		}

		@Override
		public void frameAttached(WorkbenchFrame frame) {
		}
	}

	@Override
	public void visible() {
	}
}
